﻿using CloudReporting.DataRepository;
using CloudReporting.Reports;
using Microsoft.Reporting.WebForms;
using System;
using System.Collections.Generic;
using System.Reflection;
using System.Web.Http;

namespace CloudReporting.MVC.Controllers
{
    public class ReportController : ApiController
    {
        public GetOrdersResponse Get(string category)
        {
            try
            {
                var result = new GetOrdersResponse();
                var repository = new Repository();
                result.Orders = repository.GetFakeOrdersForProductCategoryReport(category);
                result.Success = true;
                return result;
            }
            catch
            {
                //Log the error here.
                //Return a user-friendly message to the caller.
                return new GetOrdersResponse
                {
                    Success = false,
                    FailureInformation = "Could not get orders."
                };
            }
        }

        public GetOrdersAsFileResponse Get(string category, string format)
        {
            try
            {
                var response = new GetOrdersAsFileResponse();
                var data = Get(category);
                if (!data.Success)
                {
                    response.Success = false;
                    response.FailureInformation = data.FailureInformation;
                    return response;
                }

                var rv = new ReportViewer();
                rv.ProcessingMode = ProcessingMode.Local;
                rv.Reset();
                var assembly = Assembly.Load("CloudReporting.Reports");
                var stream = assembly.GetManifestResourceStream(
                "CloudReporting.Reports.OrdersForProductCategory.rdlc");
                rv.LocalReport.LoadReportDefinition(stream);
                rv.LocalReport.DataSources.Clear();
                rv.LocalReport.DataSources.Add(new ReportDataSource("DataSet1", data.Orders));
                byte[] streambytes = null;
                var mimeType = string.Empty;
                var encoding = string.Empty;
                var fileExtension = string.Empty;
                string[] streamIds = null;
                Warning[] warnings = null;
                streambytes = rv.LocalReport.Render(format, null, out mimeType,
                out encoding, out fileExtension, out streamIds, out warnings);
                rv.LocalReport.ReleaseSandboxAppDomain();  //Report viewer
                                                           //uses a lot more memory and will give out of memory exceptions
                                                           //if you don't call this.
                response.FileContents = streambytes;
                response.MimeType = mimeType;
                response.Encoding = encoding;
                response.FileName = "OrdersForProductCategory." + fileExtension;

                response.Success = data.Success;
                return response;
            }
            catch (Exception ex)
            {
                //Log the error here.
                //Return a user-friendly message to the caller.
                return new GetOrdersAsFileResponse
                {
                    Success = false,
                    FailureInformation = "Could not get orders report."
                };
            }
        }
    }

    public class GetOrdersResponse
    {
        public bool Success { get; set; } = false;
        public string FailureInformation { get; set; } = string.Empty;
        public List<OrderInformation> Orders { get; set; } = new List<OrderInformation>();
    }

    public class GetOrdersAsFileResponse
    {
        public bool Success { get; set; } = false;
        public string FailureInformation { get; set; } = string.Empty;
        public byte[] FileContents { get; set; } = null;
        public string MimeType { get; set; }
        public string Encoding { get; set; }
        public string FileName { get; set; }
    }

}